<?php
/**
 * SocialConnect project
 * @author: Patsura Dmitry https://github.com/ovr <talk@dmtry.me>
 */
declare(strict_types=1);

namespace SocialConnect\OAuth2\Provider;

use SocialConnect\Common\ArrayHydrator;
use SocialConnect\Provider\AccessTokenInterface;
use SocialConnect\Common\Entity\User;

class Steein extends \SocialConnect\OAuth2\AbstractProvider
{
    const NAME = 'steein';

    /**
     * {@inheritdoc}
     */
    public function getBaseUri()
    {
        return 'https://www.steein.ru/';
    }

    /**
     * {@inheritdoc}
     */
    public function getAuthorizeUri()
    {
        return 'https://www.steein.ru/oauth/authorize';
    }

    /**
     * {@inheritdoc}
     */
    public function getRequestTokenUri()
    {
        return 'https://www.steein.ru/oauth/token';
    }

    /**
     * @return string
     */
    public function getName()
    {
        return self::NAME;
    }

    /**
     * @return string
     */
    public function getScopeInline()
    {
        return implode(' ', $this->scope);
    }

    /**
     * {@inheritDoc}
     */
    public function prepareRequest(string $method, string $uri, array &$headers, array &$query, AccessTokenInterface $accessToken = null): void
    {
        if ($accessToken) {
            $headers['Authorization'] = "Bearer {$accessToken->getToken()}";
        }
    }

    /**
     * {@inheritdoc}
     */
    public function getIdentity(AccessTokenInterface $accessToken)
    {
        $response = $this->request('GET', 'api/v2.0/users/show', [], $accessToken);

        $hydrator = new ArrayHydrator([
            'displayName' => 'fullname',
        ]);

        /** @var User $user */
        $user = $hydrator->hydrate(new User(), $response);

        if ($response['name']) {
            if ($response['name']['first_name']) {
                $user->firstname = $response['name']['first_name'];
            }

            if ($response['name']['last_name']) {
                $user->lastname = $response['name']['last_name'];
            }
        }

        return $user;
    }
}
